@@ -1,12 +1,35 @@
 /*
- * Copyright (c) 1997 Ernest Bowen and Landon Curt Noll
- * Permission is granted to use, distribute, or modify this source,
- * provided that this copyright notice remains intact.
+ * blkcpy - general values and related routines used by the calculator
  *
- * By: Ernest Bowen and Landon Curt Noll
- *     ernie@neumann.une.edu.au and http://reality.sgi.com/chongo/
+ * Copyright (C) 1999  Landon Curt Noll and Ernest Bowen
+ *
+ * Primary author:  Landon Curt Noll
+ *
+ * Calc is open software; you can redistribute it and/or modify it under
+ * the terms of the version 2.1 of the GNU Lesser General Public License
+ * as published by the Free Software Foundation.
+ *
+ * Calc is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ * or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU Lesser General
+ * Public License for more details.
+ *
+ * A copy of version 2.1 of the GNU Lesser General Public License is
+ * distributed with calc under the filename COPYING-LGPL.  You should have
+ * received a copy with calc; if not, write to Free Software Foundation, Inc.
+ * 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
+ *
+ * @(#) $Revision: 29.3 $
+ * @(#) $Id: blkcpy.c,v 29.3 2001/04/10 22:06:46 chongo Exp $
+ * @(#) $Source: /usr/local/src/cmd/calc/RCS/blkcpy.c,v $
+ *
+ * Under source code control:	1997/04/18 20:41:26
+ * File existed as early as:	1997
+ *
+ * Share and enjoy!  :-)	http://www.isthe.com/chongo/tech/comp/calc/
  */
 
+
 #include <stdio.h>
 #include <sys/types.h>
 #include "calc.h"
@@ -547,7 +570,7 @@ copylist2list(LIST *slp, long ssi, long num, LIST *dlp, long dsi)
 int
 copyblk2file(BLOCK *sblk, long ssi, long num, FILEID id, long dsi)
 {
-	FILEIO 	*fiop;
+	FILEIO	*fiop;
 	FILE	*fp;
 	unsigned int	numw;
 
@@ -558,12 +581,12 @@ copyblk2file(BLOCK *sblk, long ssi, long num, FILEID id, long dsi)
 	if (num == 0)
 		return 0;
 
-	fiop = findid(id, 'w');
+	fiop = findid(id, TRUE);
 	if (fiop == NULL)
 		return E_COPYF1;
 	fp = fiop->fp;
 	if (id == 1 || id == 2) {
-		numw = idfputstr(id, (char *)sblk->data + ssi);  /* XXX */
+		numw = idfputstr(id, (char *)sblk->data + ssi);	 /* XXX */
 		return 0;
 	}
 	if (dsi >= 0) {
@@ -584,7 +607,7 @@ copyblk2file(BLOCK *sblk, long ssi, long num, FILEID id, long dsi)
 int
 copyfile2blk(FILEID id, long ssi, long num, BLOCK *dblk, long dsi, BOOL noreloc)
 {
-	FILEIO 	*fiop;
+	FILEIO	*fiop;
 	FILE	*fp;
 	unsigned int	numw;
 	ZVALUE	fsize;
@@ -595,7 +618,7 @@ copyfile2blk(FILEID id, long ssi, long num, BLOCK *dblk, long dsi, BOOL noreloc)
 
 	if (id < 3)			/* excludes copying from stdin */
 		return E_COPYF1;
-	fiop = findid(id, 'r');
+	fiop = findid(id, FALSE);
 	if (fiop == NULL)
 		return E_COPYF1;
 
@@ -667,12 +690,12 @@ copystr2file(STRING *str, long ssi, long num, FILEID id, long dsi)
 		return 0;
 	if ((USB32) ssi + num > len)
 		return E_COPY5;		/* Insufficient memory in str */
-	fiop = findid(id, 'w');
+	fiop = findid(id, TRUE);
 	if (fiop == NULL)
 		return E_COPYF1;
 	fp = fiop->fp;
 	if (id == 1 || id == 2) {
-		numw = idfputstr(id, str->s_str + ssi);  /* XXX */
+		numw = idfputstr(id, str->s_str + ssi);	 /* XXX */
 		return 0;
 	}
 	if (dsi >= 0) {
@@ -830,7 +853,7 @@ int
 copyostr2str(char *sstr, long ssi, long num, STRING *dstr, long dsi)
 {
 	long len;
-	char 	*c, *c1;
+	char	*c, *c1;
 
 	len = (long)strlen(sstr);
 
@@ -898,8 +921,8 @@ copyostr2blk(char *str,long ssi,long num,BLOCK *dblk,long dsi,BOOL noreloc)
  * Copying between objects that overlap will take place correctly.
  *
  * given:
- *	s1 	destination
- *	s2 	source
+ *	s1	destination
+ *	s2	source
  *	n	octet count
  *
  * returns:
@@ -959,7 +982,7 @@ copynum2blk(NUMBER *snum, long ssi, long num, BLOCK *dblk, long dsi, BOOL norelo
 	long	newsize;
 	USB8	*newdata;
 #if CALC_BYTE_ORDER == BIG_ENDIAN
-	ZVALUE	*swnum;	/* byte swapped numerator */
+	ZVALUE	*swnum; /* byte swapped numerator */
 #endif
 
 	if (ssi > snum->num.len)
