@@ -1,12 +1,40 @@
 /*
- * Copyright (c) 1995 David I. Bell
- * Permission is granted to use, distribute, or modify this source,
- * provided that this copyright notice remains intact.
+ * qtrans - transcendental functions for real numbers
  *
+ * Copyright (C) 1999  David I. Bell and Ernest Bowen
+ *
+ * Primary author:  David I. Bell
+ *
+ * Calc is open software; you can redistribute it and/or modify it under
+ * the terms of the version 2.1 of the GNU Lesser General Public License
+ * as published by the Free Software Foundation.
+ *
+ * Calc is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ * or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU Lesser General
+ * Public License for more details.
+ *
+ * A copy of version 2.1 of the GNU Lesser General Public License is
+ * distributed with calc under the filename COPYING-LGPL.  You should have
+ * received a copy with calc; if not, write to Free Software Foundation, Inc.
+ * 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
+ *
+ * @(#) $Revision: 29.2 $
+ * @(#) $Id: qtrans.c,v 29.2 2000/06/07 14:02:13 chongo Exp $
+ * @(#) $Source: /usr/local/src/cmd/calc/RCS/qtrans.c,v $
+ *
+ * Under source code control:	1990/02/15 01:48:22
+ * File existed as early as:	before 1990
+ *
+ * Share and enjoy!  :-)	http://www.isthe.com/chongo/tech/comp/calc/
+ */
+
+/*
  * Transcendental functions for real numbers.
  * These are sin, cos, exp, ln, power, cosh, sinh.
  */
 
+
 #include "qmath.h"
 
 HALF _qlgenum_[] = { 36744 };
@@ -112,9 +140,9 @@ qsincos(NUMBER *q, long bitnum, NUMBER **vs, NUMBER **vc)
 	if (m > h) {
 		zshift(cossum, -h, &qtmp1->num);
 		zbitvalue(m - h, &qtmp1->den);
-	}
-	else
+	} else {
 		zshift(cossum, - m, &qtmp1->num);
+	}
 	zfree(cossum);
 	*vc = qtmp1;
 	h = zlowbit(sinsum);
@@ -122,9 +150,9 @@ qsincos(NUMBER *q, long bitnum, NUMBER **vs, NUMBER **vc)
 	if (m > h) {
 		zshift(sinsum, -h, &qtmp2->num);
 		zbitvalue(m - h, &qtmp2->den);
-	}
-	else
+	} else {
 		zshift(sinsum, -m, &qtmp2->num);
+	}
 	zfree(sinsum);
 	*vs = qtmp2;
 	return;
@@ -145,7 +173,7 @@ qcos(NUMBER *q, NUMBER *epsilon)
 		/*NOTREACHED*/
 	}
 	if (qiszero(q))
-		return qmappr(&_qone_, epsilon, 24);
+		return qlink(&_qone_);
 	n = -qilog2(epsilon);
 	if (n < 0)
 		return qlink(&_qzero_);
@@ -283,7 +311,7 @@ qsec(NUMBER *q, NUMBER *epsilon)
 		/*NOTREACHED*/
 	}
 	if (qiszero(q))
-		return qmappr(&_qone_, epsilon, 24);
+		return qlink(&_qone_);
 	n = qilog2(epsilon);
 	k = (n > 0) ? 4 + n/2 : 4;
 	for (;;) {
@@ -378,8 +406,7 @@ qasin(NUMBER *q, NUMBER *epsilon)
 		epsilon1 = qscale(epsilon, 1L);
 		qtmp2 = qpi(epsilon1);
 		qtmp1 = qscale(qtmp2, -1L);
-	}
-	else {
+	} else {
 		epsilon1 = qscale(epsilon, -2L);
 		qtmp1 = qalloc();
 		zsquare(q->num, &qtmp1->num);
@@ -444,7 +471,7 @@ qacos(NUMBER *q, NUMBER *epsilon)
 
 /*
  * Calculate the arctangent function to the nearest or next to nearest
- * multiple of epsilon.  Algorithm uses
+ * multiple of epsilon.	 Algorithm uses
  *	atan(x) = 2 * atan(x/(1 + sqrt(1+x^2)))
  * to reduce x to a small value and then
  *	atan(x) = x - x^3/3 + ...
@@ -528,9 +555,9 @@ qatan(NUMBER *q, NUMBER *epsilon)
 	if (k) {
 		zshift(sum, -k, &qtmp->num);
 		zfree(sum);
-	}
-	else
+	} else {
 		qtmp->num = sum;
+	}
 	zbitvalue(m - 4 - k, &qtmp->den);
 	res = qmappr(qtmp, epsilon, 24L);
 	qfree(qtmp);
@@ -754,17 +781,22 @@ qexp(NUMBER *q, NUMBER *epsilon)
 		/*NOTREACHED*/
 	}
 	if (qiszero(q))
-		return qmappr(&_qone_, epsilon, 24);
+		return qlink(&_qone_);
 	tmp1 = qmul(q, &_qlge_);
-	m = qtoi(tmp1) + 1;	/* exp(q) < 2^m */
+	m = qtoi(tmp1);		/* exp(q) < 2^(m+1) or m == MAXLONG */
 	qfree(tmp1);
 
+	if (m > (1 << 30))
+		return NULL;
+
 	n = qilog2(epsilon);	/* 2^n <= epsilon < 2^(n+1) */
 	if (m < n)
 		return qlink(&_qzero_);
 	tmp1 = qqabs(q);
-	tmp2 = qexprel(tmp1, m - n + 2);
+	tmp2 = qexprel(tmp1, m - n + 1);
 	qfree(tmp1);
+	if (tmp2 == NULL)
+		return NULL;
 	if (qisneg(q)) {
 		tmp1 = qinv(tmp2);
 		qfree(tmp2);
@@ -780,6 +812,7 @@ qexp(NUMBER *q, NUMBER *epsilon)
  * Calculate the exponential function with relative error corresponding
  * to a specified number of significant bits
  * Requires *q >= 0, bitnum >= 0.
+ * This returns NULL if more than 2^30 working bits would be required.
  */
 static NUMBER *
 qexprel(NUMBER *q, long bitnum)
@@ -805,6 +838,8 @@ qexprel(NUMBER *q, long bitnum)
 		s = -h;
 	n = h + s;	/* n is number of squarings that will be required */
 	m = bitnum + n;
+	if (m > (1 << 30))
+		return NULL;
 	while (s > 0) {		/* increasing m by ilog2(s) */
 		s >>= 1;
 		m++;
@@ -842,9 +877,9 @@ qexprel(NUMBER *q, long bitnum)
 		if (zrel(ztmp1, B) >= 0) {
 			zshift(ztmp1, -m - 1, &sum);
 			k++;
-		}
-		else
+		} else {
 			zshift(ztmp1, -m, &sum);
+		}
 		zfree(ztmp1);
 	}
 	zfree(B);
@@ -929,7 +964,7 @@ qln(NUMBER *q, NUMBER *epsilon)
 
 	zcopy(pow, &sum);	/* pow is first term of sum */
 	zsquare(pow, &ztmp);
-	zshift(ztmp, -m, &mul);	/* mul is now multiplier for powers */
+	zshift(ztmp, -m, &mul); /* mul is now multiplier for powers */
 	zfree(ztmp);
 
 	d = 1;
@@ -957,8 +992,7 @@ qln(NUMBER *q, NUMBER *epsilon)
 	if (k) {
 		zshift(sum, -k, &qtmp->num);
 		zfree(sum);
-	}
-	else {
+	} else {
 		qtmp->num = sum;
 	}
 	zbitvalue(m - k - n, &qtmp->den);
@@ -988,32 +1022,28 @@ qpower(NUMBER *q1, NUMBER *q2, NUMBER *epsilon)
 		math_error("Negative power of zero");
 		/*NOTREACHED*/
 	}
-	if (qiszero(q2) || qisone(q1)) {
-		return qmappr(&_qone_, epsilon, 24L);
-	}
+	if (qiszero(q2) || qisone(q1))
+		return qlink(&_qone_);
 	if (qiszero(q1))
 		return qlink(&_qzero_);
 	if (qisneg(q1)) {
 		math_error("Negative base for qpower");
 		/*NOTREACHED*/
 	}
-	if (qisone(q2)) {
-		return qmappr(q1, epsilon, 24L);
-	}
+	if (qisone(q2))
+		return qmappr(q1, epsilon, 24);
 	if (zrel(q1->num, q1->den) < 0) {
 		q1tmp = qinv(q1);
 		q2tmp = qneg(q2);
-	}
-	else {
+	} else {
 		q1tmp = qlink(q1);
 		q2tmp = qlink(q2);
 	}
 	if (qisone(q2tmp)) {
-		tmp1 = q1tmp;
 		qfree(q2tmp);
-		tmp2 = qmappr(tmp1, epsilon, 24L);
-		qfree(tmp1);
-		return tmp2;
+		q2tmp = qmappr(q1tmp, epsilon, 24);
+		qfree(q1tmp);
+		return q2tmp;
 	}
 	m = qilog2(q1tmp);
 	n = qilog2(epsilon);
@@ -1031,8 +1061,7 @@ qpower(NUMBER *q1, NUMBER *q2, NUMBER *epsilon)
 			tmp2 = qmul(tmp1, &_qlge_);
 			m = qtoi(tmp2);
 		}
-	}
-	else {
+	} else {
 		if (m > 0) {
 			tmp1 = itoq(m + 1);
 			tmp2 = qmul(tmp1, q2tmp);
@@ -1047,6 +1076,11 @@ qpower(NUMBER *q1, NUMBER *q2, NUMBER *epsilon)
 	}
 	qfree(tmp1);
 	qfree(tmp2);
+	if (m > (1 << 30)) {
+		qfree(q1tmp);
+		qfree(q2tmp);
+		return NULL;
+	}
 	m += 1;
 	if (m < n) {
 		qfree(q1tmp);
@@ -1068,11 +1102,19 @@ qpower(NUMBER *q1, NUMBER *q2, NUMBER *epsilon)
 		tmp1 = qneg(tmp2);
 		qfree(tmp2);
 		tmp2 = qexprel(tmp1, m - n + 3);
+		if (tmp2 == NULL) {
+			qfree(tmp1);
+			return NULL;
+		}
 		qfree(tmp1);
 		tmp1 = qinv(tmp2);
-	}
-	else
+	} else {
 		tmp1 = qexprel(tmp2, m - n + 3) ;
+		if (tmp1 == NULL) {
+			qfree(tmp2);
+			return NULL;
+		}
+	}
 	qfree(tmp2);
 	tmp2 = qmappr(tmp1, epsilon, 24L);
 	qfree(tmp1);
@@ -1112,6 +1154,8 @@ qroot(NUMBER *q1, NUMBER *q2, NUMBER *epsilon)
 	tmp2 = qinv(q2);
 	tmp1 = qpower(q1, tmp2, epsilon);
 	qfree(tmp2);
+	if (tmp1 == NULL)
+		return NULL;
 	if (neg) {
 		tmp2 = qneg(tmp1);
 		qfree(tmp1);
@@ -1135,6 +1179,8 @@ qcosh(NUMBER *q, NUMBER *epsilon)
 	tmp2 = qexp(tmp1, epsilon1);
 	qfree(tmp1);
 	qfree(epsilon1);
+	if (tmp2 == NULL)
+		return NULL;
 	tmp1 = qinv(tmp2);
 	tmp3 = qqadd(tmp1, tmp2);
 	qfree(tmp1)
@@ -1164,6 +1210,8 @@ qsinh(NUMBER *q, NUMBER *epsilon)
 	tmp2 = qexp(tmp1, epsilon1);
 	qfree(tmp1);
 	qfree(epsilon1);
+	if (tmp2 == NULL)
+		return NULL;
 	tmp1 = qinv(tmp2);
 	tmp3 = qispos(q) ? qsub(tmp2, tmp1) : qsub(tmp1, tmp2);
 	qfree(tmp1)
@@ -1179,29 +1227,46 @@ qsinh(NUMBER *q, NUMBER *epsilon)
 /*
  * Calculate the hyperbolic tangent to the nearest or next to nearest
  * multiple of epsilon.
- * This is calculated using the formula:
- *	tanh(x) = (exp(2*x) - 1)/(exp(2*x) + 1).
+ * This is calculated using the formulae:
+ *	tanh(x) = 1 or -1 for very large abs(x)
+ *	tanh(x) = (+ or -)(1 - 2 * exp(2 * x))	for large abx(x)
+ *	tanh(x) = (exp(2*x) - 1)/(exp(2*x) + 1) otherwise
  */
 NUMBER *
 qtanh(NUMBER *q, NUMBER *epsilon)
 {
 	NUMBER *tmp1, *tmp2, *tmp3;
-	long n;
+	long n, m;
 
 	n = qilog2(epsilon);
 	if (n > 0 || qiszero(q))
 		return qlink(&_qzero_);
+	n = -n;
 	tmp1 = qqabs(q);
+	tmp2 = qmul(tmp1, &_qlge_);
+	m = qtoi(tmp2);		/* exp(|q|) < 2^(m+1) or m == MAXLONG */
+	qfree(tmp2);
+	if (m > 1 + n/2) {
+		qfree(tmp1);
+		return q->num.sign ? qlink(&_qnegone_) : qlink(&_qone_);
+	}
 	tmp2 = qscale(tmp1, 1);
 	qfree(tmp1);
-	tmp1 = qexprel(tmp2, 2 - n);
+	tmp1 = qexprel(tmp2, 2 + n);
 	qfree(tmp2);
-	tmp2 = qdec(tmp1);
-	tmp3 = qinc(tmp1);
-	qfree(tmp1);
-	tmp1 = qqdiv(tmp2, tmp3);
-	qfree(tmp2);
-	qfree(tmp3);
+	if (m > 1 + n/4) {
+		tmp2 = qqdiv(&_qtwo_, tmp1);
+		qfree(tmp1);
+		tmp1 = qsub(&_qone_, tmp2);
+		qfree(tmp2);
+	} else {
+		tmp2 = qdec(tmp1);
+		tmp3 = qinc(tmp1);
+		qfree(tmp1);
+		tmp1 = qqdiv(tmp2, tmp3);
+		qfree(tmp2);
+		qfree(tmp3);
+	}
 	tmp2 = qmappr(tmp1, epsilon, 24L);
 	qfree(tmp1);
 	if (qisneg(q)) {
@@ -1240,9 +1305,9 @@ qcoth(NUMBER *q, NUMBER *epsilon)
 		tmp1 = qmul(&_qlge_, tmp2);
 		k = qtoi(tmp1);
 		qfree(tmp1);
-	}
-	else
+	} else {
 		k = 2 * k;
+	}
 	k = 4 - k - n;
 	if (k < 4)
 		k = 4;
@@ -1282,7 +1347,7 @@ qsech(NUMBER *q, NUMBER *epsilon)
 		/*NOTREACHED*/
 	}
 	if (qiszero(q))
-		return qmappr(&_qone_, epsilon, 24L);
+		return qlink(&_qone_);
 
 	tmp1 = qqabs(q);
 	k = 0;
@@ -1333,9 +1398,9 @@ qcsch(NUMBER *q, NUMBER *epsilon)
 		tmp2 = qmul(&_qlge_, tmp1);
 		k = qtoi(tmp2);
 		qfree(tmp2);
-	}
-	else
+	} else {
 		k = 2 * qilog2(tmp1);
+	}
 	if (k + n >= 1) {
 		qfree(tmp1);
 		return qlink(&_qzero_);
@@ -1518,6 +1583,3 @@ qacoth(NUMBER *q, NUMBER *epsilon)
 	qfree(tmp);
 	return res;
 }
-
-
-/* END CODE */
