@@ -1,11 +1,35 @@
 /*
- * Copyright (c) 1997 David I. Bell
- * Permission is granted to use, distribute, or modify this source,
- * provided that this copyright notice remains intact.
+ * zfunc - extended precision integral arithmetic non-primitive routines
  *
- * Extended precision integral arithmetic non-primitive routines
+ * Copyright (C) 1999  David I. Bell, Landon Curt Noll and Ernest Bowen
+ *
+ * Primary author:  David I. Bell
+ *
+ * Calc is open software; you can redistribute it and/or modify it under
+ * the terms of the version 2.1 of the GNU Lesser General Public License
+ * as published by the Free Software Foundation.
+ *
+ * Calc is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ * or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU Lesser General
+ * Public License for more details.
+ *
+ * A copy of version 2.1 of the GNU Lesser General Public License is
+ * distributed with calc under the filename COPYING-LGPL.  You should have
+ * received a copy with calc; if not, write to Free Software Foundation, Inc.
+ * 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
+ *
+ * @(#) $Revision: 29.3 $
+ * @(#) $Id: zfunc.c,v 29.3 2000/07/17 15:35:49 chongo Exp $
+ * @(#) $Source: /usr/local/src/cmd/calc/RCS/zfunc.c,v $
+ *
+ * Under source code control:	1990/02/15 01:48:27
+ * File existed as early as:	before 1990
+ *
+ * Share and enjoy!  :-)	http://www.isthe.com/chongo/tech/comp/calc/
  */
 
+
 #include "zmath.h"
 
 ZVALUE _tenpowers_[TEN_MAX+1];		/* table of 10^2^n */
@@ -27,7 +51,7 @@ zfact(ZVALUE z, ZVALUE *dest)
 		math_error("Negative argument for factorial");
 		/*NOTREACHED*/
 	}
-	if (zge24b(z)) {
+	if (zge31b(z)) {
 		math_error("Very large factorial");
 		/*NOTREACHED*/
 	}
@@ -37,7 +61,7 @@ zfact(ZVALUE z, ZVALUE *dest)
 	res = _one_;
 	/*
 	 * Multiply numbers together, but squeeze out all powers of two.
-	 * We will put them back in at the end.  Also collect multiple
+	 * We will put them back in at the end.	 Also collect multiple
 	 * numbers together until there is a risk of overflow.
 	 */
 	for (; n > 1; n--) {
@@ -84,7 +108,7 @@ zperm(ZVALUE z1, ZVALUE z2, ZVALUE *res)
 		math_error("Second arg larger than first in permutation");
 		/*NOTREACHED*/
 	}
-	if (zge24b(z2)) {
+	if (zge31b(z2)) {
 		math_error("Very large permutation");
 		/*NOTREACHED*/
 	}
@@ -103,58 +127,104 @@ zperm(ZVALUE z1, ZVALUE z2, ZVALUE *res)
 	*res = ans;
 }
 
-
 /*
- * Compute the combinatorial function  M! / ( N! * (M - N)! ).
+ * docomb evaluates binomial coefficient when z1 >= 0, z2 >= 0
  */
-void
-zcomb(ZVALUE z1, ZVALUE z2, ZVALUE *res)
+static int
+docomb(ZVALUE z1, ZVALUE z2, ZVALUE *res)
 {
 	ZVALUE ans;
 	ZVALUE mul, div, temp;
 	FULL count, i;
+#if BASEB == 16
 	HALF dh[2];
+#else
+	HALF dh[1];
+#endif
 
-	if (zisneg(z1) || zisneg(z2)) {
-		math_error("Negative argument for combinatorial");
-		/*NOTREACHED*/
-	}
+	if (zrel(z2, z1) > 0)
+		return 0;
 	zsub(z1, z2, &temp);
-	if (zisneg(temp)) {
+
+	if (zge31b(z2) && zge31b(temp)) {
 		zfree(temp);
-		math_error("Second arg larger than first for combinatorial");
-		/*NOTREACHED*/
+		return -2;
 	}
-	if (zge24b(z2) && zge24b(temp)) {
-		zfree(temp);
-		math_error("Very large combinatorial");
-		/*NOTREACHED*/
-	}
-	count = ztofull(z2);
-	i = ztofull(temp);
-	if (zge24b(z2) || (!zge24b(temp) && (i < count)))
-		count = i;
+	if (zrel(temp, z2) < 0)
+		count = ztofull(temp);
+	else
+		count = ztofull(z2);
 	zfree(temp);
-	mul = z1;
+	if (count == 0)
+		return 1;
+	if (count == 1)
+		return 2;
 	div.sign = 0;
 	div.v = dh;
-	ans = _one_;
-	for (i = 1; i <= count; i++) {
+	div.len = 1;
+	zcopy(z1, &mul);
+	zcopy(z1, &ans);
+	for (i = 2; i <= count; i++) {
+#if BASEB == 16
 		dh[0] = (HALF)(i & BASE1);
 		dh[1] = (HALF)(i >> BASEB);
 		div.len = 1 + (dh[1] != 0);
+#else
+		dh[0] = (HALF) i;
+#endif
+		zsub(mul, _one_, &temp);
+		zfree(mul);
+		mul = temp;
 		zmul(ans, mul, &temp);
 		zfree(ans);
 		zquo(temp, div, &ans, 0);
 		zfree(temp);
-		zsub(mul, _one_, &temp);
-		if (mul.v != z1.v)
-			zfree(mul);
-		mul = temp;
 	}
-	if (mul.v != z1.v)
-		zfree(mul);
+	zfree(mul);
 	*res = ans;
+	return 3;
+}
+
+/*
+ * Compute the combinatorial function  M! / ( N! * (M - N)! ).
+ * Returns 0 if result is 0
+*	   1      1
+*	   2      z1
+*	  -1	  -1
+*	  -2	if too complicated
+*	   3	result stored at res
+ */
+int
+zcomb(ZVALUE z1, ZVALUE z2, ZVALUE *res)
+{
+	ZVALUE z3, z4;
+	int r;
+
+	if (z2.sign || (!z1.sign && zrel(z2, z1) > 0))
+		return 0;
+	if (zisone(z2))
+		return 2;
+	if (z1.sign) {
+		z1.sign = 0;
+		zsub(z1, _one_, &z3);
+		zadd(z3, z2, &z4);
+		zfree(z3);
+		r = docomb(z4, z2, res);
+		if (r == 2) {
+			*res = z4;
+			r = 3;
+		}
+		else
+			zfree(z4);
+		if (z2.v[0] & 1) {
+			if (r == 1)
+				r = -1;
+			if (r == 3)
+				res->sign = 1;
+		}
+		return r;
+	}
+	return docomb(z1, z2, res);
 }
 
 
@@ -450,7 +520,7 @@ ztenpow(long power, ZVALUE *res)
 
 /*
  * Calculate modular inverse suppressing unnecessary divisions.
- * This is based on the Euclidian algorithm for large numbers.
+ * This is based on the Euclidean algorithm for large numbers.
  * (Algorithm X from Knuth Vol 2, section 4.5.2. and exercise 17)
  * Returns TRUE if there is no solution because the numbers
  * are not relatively prime.
@@ -458,7 +528,7 @@ ztenpow(long power, ZVALUE *res)
 BOOL
 zmodinv(ZVALUE u, ZVALUE v, ZVALUE *res)
 {
- 	FULL	q1, q2, ui3, vi3, uh, vh, A, B, C, D, T;
+	FULL	q1, q2, ui3, vi3, uh, vh, A, B, C, D, T;
 	ZVALUE	u2, u3, v2, v3, qz, tmp1, tmp2, tmp3;
 
 	v.sign = 0;
@@ -472,7 +542,7 @@ zmodinv(ZVALUE u, ZVALUE v, ZVALUE *res)
 
 	/*
 	 * Loop here while the size of the numbers remain above
-	 * the size of a HALF.  Throughout this loop u3 >= v3.
+	 * the size of a HALF.	Throughout this loop u3 >= v3.
 	 */
 	while ((u3.len > 1) && !ziszero(v3)) {
 		vh = 0;
@@ -481,11 +551,11 @@ zmodinv(ZVALUE u, ZVALUE v, ZVALUE *res)
 		if (v3.len == u3.len)
 			vh = v3.v[v3.len - 1];
 #else
- 		uh = (((FULL) u3.v[u3.len - 1]) << BASEB) + u3.v[u3.len - 2];
- 		if ((v3.len + 1) >= u3.len)
- 			vh = v3.v[v3.len - 1];
-  		if (v3.len == u3.len)
- 			vh = (vh << BASEB) + v3.v[v3.len - 2];
+		uh = (((FULL) u3.v[u3.len - 1]) << BASEB) + u3.v[u3.len - 2];
+		if ((v3.len + 1) >= u3.len)
+			vh = v3.v[v3.len - 1];
+		if (v3.len == u3.len)
+			vh = (vh << BASEB) + v3.v[v3.len - 2];
 #endif
 		A = 1;
 		B = 0;
@@ -611,61 +681,6 @@ zmodinv(ZVALUE u, ZVALUE v, ZVALUE *res)
 }
 
 
-#if 0
-/*
- * Approximate the quotient of two integers by another set of smaller
- * integers.  This uses continued fractions to determine the smaller set.
- */
-void
-zapprox(ZVALUE z1, ZVALUE z2, ZVALUE *res1, ZVALUE *res2)
-{
-	int sign;
-	ZVALUE u1, v1, u3, v3, q, t1, t2, t3;
-
-	sign = ((z1.sign != 0) ^ (z2.sign != 0));
-	z1.sign = 0;
-	z2.sign = 0;
-	v3 = z2;
-	u3 = z1;
-	u1 = _one_;
-	v1 = _zero_;
-	while (!ziszero(v3)) {
-		zdiv(u3, v3, &q, &t1, 0);
-		zmul(v1, q, &t2);
-		zsub(u1, t2, &t3);
-		zfree(q);
-		zfree(t2);
-		zfree(u1);
-		if ((u3.v != z1.v) && (u3.v != z2.v))
-			zfree(u3);
-		u1 = v1;
-		u3 = v3;
-		v1 = t3;
-		v3 = t1;
-	}
-	if (!zisunit(u3)) {
-		math_error("Non-relativly prime numbers for approx");
-		/*NOTREACHED*/
-	}
-	if ((u3.v != z1.v) && (u3.v != z2.v))
-		zfree(u3);
-	if ((v3.v != z1.v) && (v3.v != z2.v))
-		zfree(v3);
-	zfree(v1);
-	zmul(u1, z1, &t1);
-	zsub(t1, _one_, &t2);
-	zfree(t1);
-	zquo(t2, z2, &t1, 0);
-	zfree(t2);
-	u1.sign = (BOOL)sign;
-	t1.sign = 0;
-	*res1 = t1;
-	*res2 = u1;
-}
-#endif
-
-
-
 /*
  * Compute the greatest common divisor of a pair of integers.
  */
@@ -754,7 +769,7 @@ zgcd(ZVALUE z1, ZVALUE z2, ZVALUE *res)
 		b0 = A;
 		m = 1;
 		a0 = B;
-    		if (m == 1) {				/* a has one digit */
+		if (m == 1) {				/* a has one digit */
 			v = *a0;
 			if (v > 1) {			/* Euclid's algorithm */
 				b = b0 + n;
@@ -784,8 +799,9 @@ zgcd(ZVALUE z1, ZVALUE z2, ZVALUE *res)
 				*a++ = (HALF) f;
 			}
 			if (f >>= BASEB) {len++; *a = (HALF) f;}
+		} else {
+			memcpy(gcd.v + o, b0, n * sizeof(HALF));
 		}
-		else memcpy(gcd.v + o, b0, n * sizeof(HALF));
 		gcd.len = len;
 		gcd.sign = 0;
 		freeh(A);
@@ -795,7 +811,7 @@ zgcd(ZVALUE z1, ZVALUE z2, ZVALUE *res)
 	}
 
 	u = B[n-1];				/* Bit count for b */
-	k = (n  - 1) * BASEB;
+	k = (n	- 1) * BASEB;
 	while (u >>= 1) k++;
 
 	needw = TRUE;
@@ -880,7 +896,7 @@ zgcd(ZVALUE z1, ZVALUE z2, ZVALUE *res)
 				m--;
 				a0++;
 			}
-			if (f) { 		/* a - g * b < 0 */
+			if (f) {		/* a - g * b < 0 */
 				while (m > 1 && a0[m-1] == BASE1) m--;
 				*a0 = - *a0;
 				a = a0;
@@ -962,9 +978,12 @@ zgcd(ZVALUE z1, ZVALUE z2, ZVALUE *res)
 			*a++ = (HALF) f;
 			f >>= BASEB;
 		}
-		if (f) {len++; *a = (HALF) f;}
+		if (f) {
+			len++; *a = (HALF) f;
+		}
+	} else {
+		memcpy(gcd.v + o, b0, n * sizeof(HALF));
 	}
-	else memcpy(gcd.v + o, b0, n * sizeof(HALF));
 	gcd.len = len;
 	gcd.sign = 0;
 	freeh(A);
@@ -1002,11 +1021,11 @@ zrelprime(ZVALUE z1, ZVALUE z2)
 
 	z1.sign = 0;
 	z2.sign = 0;
-	if (ziseven(z1) && ziseven(z2))	/* false if both even */
+	if (ziseven(z1) && ziseven(z2)) /* false if both even */
 		return FALSE;
-	if (zisunit(z1) || zisunit(z2))	/* true if either is a unit */
+	if (zisunit(z1) || zisunit(z2)) /* true if either is a unit */
 		return TRUE;
-	if (ziszero(z1) || ziszero(z2))	/* false if either is zero */
+	if (ziszero(z1) || ziszero(z2)) /* false if either is zero */
 		return FALSE;
 	if (zistwo(z1) || zistwo(z2))	/* true if either is two */
 		return TRUE;
@@ -1048,108 +1067,76 @@ zrelprime(ZVALUE z1, ZVALUE z2)
 
 
 /*
- * Compute the log of one number base another, to the closest integer.
- * This is the largest integer which when the second number is raised to it,
- * the resulting value is less than or equal to the first number.
+ * Compute the integer floor of the log of an integer to a specified base.
+ * The signs of the integers and base are ignored.
  * Example:  zlog(123456, 10) = 5.
  */
 long
-zlog(ZVALUE z1, ZVALUE z2)
+zlog(ZVALUE z, ZVALUE base)
 {
-	register ZVALUE *zp;		/* current square */
+	ZVALUE *zp;			/* current square */
 	long power;			/* current power */
-	long worth;			/* worth of current square */
-	ZVALUE val;			/* current value of power */
 	ZVALUE temp;			/* temporary */
 	ZVALUE squares[32];		/* table of squares of base */
 
+	/* ignore signs */
+
+	z.sign = 0;
+	base.sign = 0;
+
 	/*
-	 * Make sure that the numbers are > 0 and the base is > 1
+	 * Make sure that the numbers are nonzero and the base is > 1
 	 */
-	if (zislezero(z1) || zisleone(z2)) {
-		math_error("Bad arguments for log");
+	if (ziszero(z) || ziszero(base) || zisone(base)) {
+		math_error("Zero or too small argument argument for zlog!!!");
 		/*NOTREACHED*/
 	}
 
 	/*
-	 * Reject trivial cases.
+	 * Some trivial cases.
 	 */
-	if (z1.len < z2.len)
-		return 0;
-	if ((z1.len == z2.len) && (z1.v[z1.len-1] < z2.v[z2.len-1]))
-		return 0;
-	power = zrel(z1, z2);
+	power = zrel(z, base);
 	if (power <= 0)
 		return (power + 1);
-	/*
-	 * Handle any power of two special.
-	 */
-	if (zisonebit(z2))
-		return (zhighbit(z1) / zlowbit(z2));
-	/*
-	 * Handle base 10 special
-	 */
-	if ((z2.len == 1) && (*z2.v == 10))
-		return zlog10(z1);
+
+	/* base - power of two */
+	if (zisonebit(base))
+		return (zhighbit(z) / zlowbit(base));
+
+	/* base = 10 */
+	if (base.len == 1 && base.v[0] == 10)
+		return zlog10(z);
 	/*
 	 * Now loop by squaring the base each time, and see whether or
 	 * not each successive square is still smaller than the number.
 	 */
-	worth = 1;
 	zp = &squares[0];
-	*zp = z2;
-	while (((zp->len * 2) - 1) <= z1.len) {	/* while square not too large */
+	*zp = base;
+	while (zp->len * 2 - 1 <= z.len  && zrel(z, *zp) > 0) {
+		/* while square not too large */
 		zsquare(*zp, zp + 1);
 		zp++;
-		worth *= 2;
 	}
+
 	/*
-	 * Now back down the squares, and multiply them together to see
-	 * exactly how many times the base can be raised by.
+	 * Now back down the squares,
 	 */
-	val = _one_;
 	power = 0;
-
-	/*
-	 * We prevent the zp pointer from walking behind squares
-	 * by stopping one short of the end and running the loop one
-	 * more time.
-	 *
-	 * We could stop the loop with just zp >= squares, but stopping
-	 * short and running the loop one last time manually helps make 
-	 * code checkers such as insure happy.
-	 */
-	for (; zp > squares; zp--, worth /= 2) {
-		if ((val.len + zp->len - 1) <= z1.len) {
-			zmul(val, *zp, &temp);
-			if (zrel(z1, temp) >= 0) {
-				zfree(val);
-				val = temp;
-				power += worth;
-			} else {
-				zfree(temp);
-			}
+	for (; zp > squares; zp--) {
+		if (zrel(z, *zp) >= 0) {
+			zquo(z, *zp, &temp, 0);
+			if (power)
+				zfree(z);
+			z = temp;
+			power++;
 		}
-		if (zp != squares)
-			zfree(*zp);
+		zfree(*zp);
+		power <<= 1;
 	}
-	/* run the loop manually one last time */
-	if (zp == squares) {
-		if ((val.len + zp->len - 1) <= z1.len) {
-			zmul(val, *zp, &temp);
-			if (zrel(z1, temp) >= 0) {
-				zfree(val);
-				val = temp;
-				power += worth;
-			} else {
-				zfree(temp);
-			}
-		}
-		if (zp != squares)
-			zfree(*zp);
-	}
-
-	zfree(val);
+	if (zrel(z, *zp) >= 0)
+		power++;
+	if (power > 1)
+		zfree(z);
 	return power;
 }
 
@@ -1162,70 +1149,50 @@ zlog10(ZVALUE z)
 {
 	register ZVALUE *zp;		/* current square */
 	long power;			/* current power */
-	long worth;			/* worth of current square */
-	ZVALUE val;			/* current value of power */
 	ZVALUE temp;			/* temporary */
 
-	if (!zispos(z)) {
-		math_error("Non-positive number for log10");
+	if (ziszero(z)) {
+		math_error("Zero argument argument for zlog10");
 		/*NOTREACHED*/
 	}
+	/* Ignore sign of z */
+	z.sign = 0;
+
 	/*
 	 * Loop by squaring the base each time, and see whether or
 	 * not each successive square is still smaller than the number.
 	 */
-	worth = 1;
 	zp = &_tenpowers_[0];
 	*zp = _ten_;
 	while (((zp->len * 2) - 1) <= z.len) {	/* while square not too large */
+		if (zp >= &_tenpowers_[TEN_MAX]) {
+			math_error("Maximum storable power of 10 reached!");
+			/*NOTREACHED*/
+		}
 		if (zp[1].len == 0)
 			zsquare(*zp, zp + 1);
 		zp++;
-		worth *= 2;
 	}
 	/*
 	 * Now back down the squares, and multiply them together to see
 	 * exactly how many times the base can be raised by.
 	 */
-	val = _one_;
 	power = 0;
 
-	/*
-	 * We prevent the zp pointer from walking behind _tenpowers_
-	 * by stopping one short of the end and running the loop one
-	 * more time.
-	 *
-	 * We could stop the loop with just zp >= _tenpowers_, but stopping
-	 * short and running the loop one last time manually helps make 
-	 * code checkers such as insure happy.
-	 */
-	for (; zp > _tenpowers_; zp--, worth /= 2) {
-		if ((val.len + zp->len - 1) <= z.len) {
-			zmul(val, *zp, &temp);
-			if (zrel(z, temp) >= 0) {
-				zfree(val);
-				val = temp;
-				power += worth;
-			} else {
-				zfree(temp);
-			}
+	for (; zp > _tenpowers_; zp--) {
+		if (zrel(z, *zp) >= 0) {
+			zquo(z, *zp, &temp, 0);
+			if (power)
+				zfree(z);
+			z = temp;
+			power++;
 		}
+		power <<= 1;
 	}
-	/* run the loop manually one last time */
-	if (zp == _tenpowers_) {
-		if ((val.len + zp->len - 1) <= z.len) {
-			zmul(val, *zp, &temp);
-			if (zrel(z, temp) >= 0) {
-				zfree(val);
-				val = temp;
-				power += worth;
-			} else {
-				zfree(temp);
-			}
-		}
-	}
-
-	zfree(val);
+	if (zrel(z, *zp) >= 0)
+		power++;
+	if (power > 1)
+		zfree(z);
 	return power;
 }
 
@@ -1250,7 +1217,7 @@ zdivcount(ZVALUE z1, ZVALUE z2)
 
 
 /*
- * Remove all occurences of the specified factor from a number.
+ * Remove all occurrences of the specified factor from a number.
  * Also returns the number of factors removed as a function return value.
  * Example:  zfacrem(540, 3, &x) returns 3 and sets x to 20.
  */
@@ -1315,7 +1282,7 @@ zfacrem(ZVALUE z1, ZVALUE z2, ZVALUE *rem)
 	worth = 1;
 	zp = &squares[0];
 	*zp = z2;
-	while (((zp->len * 2) - 1) <= z1.len) {	/* while square not too large */
+	while (((zp->len * 2) - 1) <= z1.len) { /* while square not too large */
 		zsquare(*zp, &temp1);
 		zdiv(z1, temp1, &temp2, &temp3, 0);
 		if (!ziszero(temp3)) {
@@ -1342,7 +1309,7 @@ zfacrem(ZVALUE z1, ZVALUE z2, ZVALUE *rem)
 	 * more time.
 	 *
 	 * We could stop the loop with just zp >= squares, but stopping
-	 * short and running the loop one last time manually helps make 
+	 * short and running the loop one last time manually helps make
 	 * code checkers such as insure happy.
 	 */
 	for (; zp > squares; zp--, worth /= 2) {
@@ -1384,29 +1351,47 @@ zfacrem(ZVALUE z1, ZVALUE z2, ZVALUE *rem)
 
 /*
  * Keep dividing a number by the gcd of it with another number until the
- * result is relatively prime to the second number.
+ * result is relatively prime to the second number.  Returns the number
+ * of divisions made, and if this is positive, stores result at res.
  */
-void
+long
 zgcdrem(ZVALUE z1, ZVALUE z2, ZVALUE *res)
 {
 	ZVALUE tmp1, tmp2;
+	long count, onecount;
+	long sh;
 
+	if (ziszero(z1) || ziszero(z2)) {
+		math_error("Zero argument in call to zgcdrem!!!");
+		/*NOTREACHED*/
+	}
 	/*
 	 * Begin by taking the gcd for the first time.
 	 * If the number is already relatively prime, then we are done.
 	 */
 	z1.sign = 0;
 	z2.sign = 0;
-	zgcd(z1, z2, &tmp1);
-	if (zisunit(tmp1) || ziszero(tmp1)) {
-		res->len = z1.len;
-		res->v = alloc(z1.len);
-		res->sign = 0;
-		zcopyval(z1, *res);
-		zfree(tmp1);
-		return;
+	if (zisone(z2))
+		return 0;
+	if (zisonebit(z2)) {
+		sh = zlowbit(z1);
+		if (sh == 0)
+			return 0;
+		zshift(z1, -sh, res);
+		return 1 + (sh - 1)/zlowbit(z2);
 	}
+	if (zisonebit(z1)) {
+		if (zisodd(z2))
+			return 0;
+		*res = _one_;
+		return zlowbit(z1);
+	}
+
+	zgcd(z1, z2, &tmp1);
+	if (zisunit(tmp1) || ziszero(tmp1))
+		return 0;
 	zequo(z1, tmp1, &tmp2);
+	count = 1;
 	z1 = tmp2;
 	z2 = tmp1;
 	/*
@@ -1414,15 +1399,18 @@ zgcdrem(ZVALUE z1, ZVALUE z2, ZVALUE *res)
 	 * the gcd becomes one.
 	 */
 	while (!zisunit(z2)) {
-		(void) zfacrem(z1, z2, &tmp1);
-		zfree(z1);
-		z1 = tmp1;
+		onecount = zfacrem(z1, z2, &tmp1);
+		if (onecount) {
+			count += onecount;
+			zfree(z1);
+			z1 = tmp1;
+		}
 		zgcd(z1, z2, &tmp1);
 		zfree(z2);
 		z2 = tmp1;
 	}
-	zfree(z2);
 	*res = z1;
+	return count;
 }
 
 
@@ -1450,7 +1438,7 @@ zdigits(ZVALUE z1)
 
 /*
  * Return the single digit at the specified decimal place of a number,
- * where 0 means the rightmost digit.  Example:  zdigit(1234, 1) = 3.
+ * where 0 means the rightmost digit.  Example:	 zdigit(1234, 1) = 3.
  */
 long
 zdigit(ZVALUE z1, long n)
@@ -1482,8 +1470,8 @@ zdigit(ZVALUE z1, long n)
  * z is to be a nonnegative integer
  * If z is the square of a integer stores at dest the square root of z;
  *	otherwise stores at z an integer differing from the square root
- *	by less than 1.  Returns the sign of the true square root minus
- *	the calculated integer.  Type of rounding is determined by
+ *	by less than 1.	 Returns the sign of the true square root minus
+ *	the calculated integer.	 Type of rounding is determined by
  *	rnd as follows: rnd = 0 gives round down, rnd = 1
  *	rounds up, rnd = 8 rounds to even integer, rnd = 9 rounds to odd
  *	integer, rnd = 16 rounds to nearest integer.
@@ -1535,8 +1523,7 @@ zsqrt(ZVALUE z, ZVALUE *dest, long rnd)
 			f = A[1];
 		g = (FULL) A[0] << (j + BASEB);
 		d = e = topbit = (FULL)1 << (k - 1);
-	}
-	else {
+	} else {
 		if (j)
 			f = (FULL) A[m-1] << (j + BASEB) | (FULL) A[m-2] << j |
 				A[m-3] >> k;
@@ -1609,8 +1596,7 @@ zsqrt(ZVALUE z, ZVALUE *dest, long rnd)
 		A[m1 - 4] = (HALF)f;
 		m = m1 - 2;
 		k1 = k + 1;
-	}
-	else {
+	} else {
 		A[m1 - 1] = 1;
 		A[m1 - 2] = (HALF)(e >> (BASEB - 1));
 		A[m1 - 3] = ((HALF)(e << 1) | (HALF)(s > 0));
@@ -1667,9 +1653,9 @@ zsqrt(ZVALUE z, ZVALUE *dest, long rnd)
 					a[1] |= 1;
 			}
 			*a = ((HALF)(x << 1) | (HALF)(u > 0));
-		}
-		else
+		} else {
 			*a = u;
+		}
 		m--;
 		if (*--a == u) {
 			while (m > 1 && *--a == u)
@@ -1742,9 +1728,9 @@ done:	if (s == 0) {
 			n++;
 			*a = 1;
 		}
-	}
-	else
+	} else {
 		remsign = 1;
+	}
 	sqrt.v = alloc(n);
 	sqrt.len = n;
 	sqrt.sign = 0;
@@ -1802,7 +1788,7 @@ zroot(ZVALUE z1, ZVALUE z2, ZVALUE *dest)
 	sival.ivalue = k - 1;
 	k1.v = &sival.silow;
 	/* ignore Saber-C warning #112 - get ushort from uint */
-	/* 	  ok to ignore on name zroot`sival */
+	/*	  ok to ignore on name zroot`sival */
 	k1.len = 1 + (sival.sihigh != 0);
 	k1.sign = 0;
 	z1.sign = 0;
@@ -1849,7 +1835,7 @@ zroot(ZVALUE z1, ZVALUE z2, ZVALUE *dest)
 			old.len = ztry.len;
 			zcopyval(ztry, old);
 		}
-		/* average current try and quotent for the new try */
+		/* average current try and quotient for the new try */
 		zmul(ztry, k1, &temp);
 		zfree(ztry);
 		zadd(quo, temp, &temp2);
@@ -1903,5 +1889,3 @@ zissquare(ZVALUE z)
 	zfree(tmp);
 	return (n ? TRUE : FALSE);
 }
-
-/* END CODE */
