@@ -1,25 +1,47 @@
 /*
- * Copyright (c) 1996 David I. Bell
- * Permission is granted to use, distribute, or modify this source,
- * provided that this copyright notice remains intact.
+ * zio - scanf and printf routines for arbitrary precision integers
  *
- * Scanf and printf routines for arbitrary precision integers.
+ * Copyright (C) 1999  David I. Bell
+ *
+ * Calc is open software; you can redistribute it and/or modify it under
+ * the terms of the version 2.1 of the GNU Lesser General Public License
+ * as published by the Free Software Foundation.
+ *
+ * Calc is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ * or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU Lesser General
+ * Public License for more details.
+ *
+ * A copy of version 2.1 of the GNU Lesser General Public License is
+ * distributed with calc under the filename COPYING-LGPL.  You should have
+ * received a copy with calc; if not, write to Free Software Foundation, Inc.
+ * 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
+ *
+ * @(#) $Revision: 29.3 $
+ * @(#) $Id: zio.c,v 29.3 2000/07/17 15:35:49 chongo Exp $
+ * @(#) $Source: /usr/local/src/cmd/calc/RCS/zio.c,v $
+ *
+ * Under source code control:	1993/07/30 19:42:48
+ * File existed as early as:	1993
+ *
+ * Share and enjoy!  :-)	http://www.isthe.com/chongo/tech/comp/calc/
  */
 
+
 #include <stdio.h>
 #include "config.h"
 #include "zmath.h"
 #include "args.h"
 
 
-#define	OUTBUFSIZE	200		/* realloc size for output buffers */
+#define OUTBUFSIZE	200		/* realloc size for output buffers */
 
-#define	PUTCHAR(ch)		math_chr(ch)
-#define	PUTSTR(str)		math_str(str)
-#define	PRINTF1(fmt, a1)		math_fmt(fmt, a1)
-#define	PRINTF2(fmt, a1, a2)		math_fmt(fmt, a1, a2)
-#define	PRINTF3(fmt, a1, a2, a3)	math_fmt(fmt, a1, a2, a3)
-#define	PRINTF4(fmt, a1, a2, a3, a4)	math_fmt(fmt, a1, a2, a3, a4)
+#define PUTCHAR(ch)		math_chr(ch)
+#define PUTSTR(str)		math_str(str)
+#define PRINTF1(fmt, a1)		math_fmt(fmt, a1)
+#define PRINTF2(fmt, a1, a2)		math_fmt(fmt, a1, a2)
+#define PRINTF3(fmt, a1, a2, a3)	math_fmt(fmt, a1, a2, a3)
+#define PRINTF4(fmt, a1, a2, a3, a4)	math_fmt(fmt, a1, a2, a3, a4)
 
 
 /*
@@ -176,7 +198,7 @@ math_flush(void)
 /*
  * Divert further output so that it is saved into a string that will be
  * returned later when the diversion is completed.  The current state of
- * output is remembered for later restoration.  Diversions can be nested.
+ * output is remembered for later restoration.	Diversions can be nested.
  * Output diversion is only intended for saving output to "stdout".
  */
 void
@@ -296,10 +318,10 @@ math_setmode(int newmode)
  * Set the number of digits for float or exponential output.
  * This also returns the previous number of digits.
  */
-long
-math_setdigits(long newdigits)
+LEN
+math_setdigits(LEN newdigits)
 {
-	long olddigits;
+	LEN olddigits;
 
 	if (newdigits < 0) {
 		math_error("Setting illegal number of digits");
@@ -314,9 +336,9 @@ math_setdigits(long newdigits)
 /*
  * Print an integer value as a hex number.
  * Width is the number of columns to print the number in, including the
- * sign if required.  If zero, no extra output is done.  If positive,
+ * sign if required.  If zero, no extra output is done.	 If positive,
  * leading spaces are typed if necessary. If negative, trailing spaces are
- * typed if necessary.  The special characters 0x appear to indicate the
+ * typed if necessary.	The special characters 0x appear to indicate the
  * number is hex.
  */
 /*ARGSUSED*/
@@ -394,7 +416,7 @@ zprintb(ZVALUE z, long width)
 	didprint = 0;
 	PUTSTR("0b");
 	while (len-- >= 0) {
-		val = *hp--;
+		val = ((len >= 0) ? *hp-- : *hp);
 		mask = ((HALF)1 << (BASEB - 1));
 		while (mask) {
 			ch = '0' + ((mask & val) != 0);
@@ -457,7 +479,7 @@ zprinto(ZVALUE z, long width)
 		num4 = (hp[-2] & 0xffffff);
 		if (num1) {
 			PRINTF4("0%lo%08lo%08lo%08lo",
- 			    (PRINT) num1, (PRINT) num2,
+			    (PRINT) num1, (PRINT) num2,
 			    (PRINT) num3, (PRINT) num4);
 		} else {
 			PRINTF3("0%lo%08lo%08lo",
@@ -481,15 +503,17 @@ zprinto(ZVALUE z, long width)
 		break;
 	}
 	len -= rem;
-	hp -= rem;
-	while (len > 0) {	/* finish in groups of 3 words */
-		PRINTF4("%08lo%08lo%08lo%08lo",
-		    (PRINT) ((hp[0]) >> 8),
-		    (PRINT) (((hp[0] & 0xff) << 16) + (hp[-1] >> 16)),
-		    (PRINT) (((hp[-1] & 0xffff) << 8) + (hp[-2] >> 24)),
-		    (PRINT) (hp[-2] & 0xffffff));
-		hp -= 3;
-		len -= 3;
+	if (len > 0) {
+		hp -= rem;
+		while (len > 0) {	/* finish in groups of 3 words */
+			PRINTF4("%08lo%08lo%08lo%08lo",
+			    (PRINT) ((hp[0]) >> 8),
+			    (PRINT) (((hp[0] & 0xff) << 16) + (hp[-1] >> 16)),
+			    (PRINT) (((hp[-1] & 0xffff) << 8) + (hp[-2] >> 24)),
+			    (PRINT) (hp[-2] & 0xffffff));
+			hp -= 3;
+			len -= 3;
+		}
 	}
 #else
 	switch (rem) {	/* handle odd amounts first */
@@ -513,13 +537,15 @@ zprinto(ZVALUE z, long width)
 		PRINTF1("0%lo", num2);
 	}
 	len -= rem;
-	hp -= rem;
-	while (len > 0) {	/* finish in groups of 3 halfwords */
-		PRINTF2("%08lo%08lo",
-		    ((((FULL) hp[0]) << 8) + (((FULL) hp[-1]) >> 8)),
-		    ((((FULL) (hp[-1] & 0xff)) << 16) + ((FULL) hp[-2])));
-		hp -= 3;
-		len -= 3;
+	if (len > 0) {
+		hp -= rem;
+		while (len > 0) {	/* finish in groups of 3 halfwords */
+			PRINTF2("%08lo%08lo",
+			    ((((FULL) hp[0]) << 8) + (((FULL) hp[-1]) >> 8)),
+			    ((((FULL) (hp[-1] & 0xff))<<16) + ((FULL) hp[-2])));
+			hp -= 3;
+			len -= 3;
+		}
 	}
 #endif
 }
@@ -532,10 +558,10 @@ zprinto(ZVALUE z, long width)
  * Decimals supplies number of decimal places to print, with a decimal
  * point at the right location, with zero meaning no decimal point.
  * Width is the number of columns to print the number in, including the
- * decimal point and sign if required.  If zero, no extra output is done.
+ * decimal point and sign if required.	If zero, no extra output is done.
  * If positive, leading spaces are typed if necessary. If negative, trailing
  * spaces are typed if necessary.  As examples of the effects of these values,
- * (345,0,0) = "345", (345,2,0) = "3.45", (345,5,8) = "  .00345".
+ * (345,0,0) = "345", (345,2,0) = "3.45", (345,5,8) = "	 .00345".
  *
  * given:
  *	z		number to be printed
@@ -651,7 +677,7 @@ zprintval(ZVALUE z, long decimals, long width)
 /*
  * Read an integer value in decimal, hex, octal, or binary.
  * Hex numbers are indicated by a leading "0x", binary with a leading "0b",
- * and octal by a leading "0".  Periods are skipped over, but any other
+ * and octal by a leading "0".	Periods are skipped over, but any other
  * extraneous character stops the scan.
  */
 void
