@@ -1,8 +1,33 @@
 /*
- * Copyright (c) 1997 David I. Bell
- * Permission is granted to use, distribute, or modify this source,
- * provided that this copyright notice remains intact.
+ * zmath - declarations for extended precision integer arithmetic
  *
+ * Copyright (C) 1999  David I. Bell
+ *
+ * Calc is open software; you can redistribute it and/or modify it under
+ * the terms of the version 2.1 of the GNU Lesser General Public License
+ * as published by the Free Software Foundation.
+ *
+ * Calc is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ * or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU Lesser General
+ * Public License for more details.
+ *
+ * A copy of version 2.1 of the GNU Lesser General Public License is
+ * distributed with calc under the filename COPYING-LGPL.  You should have
+ * received a copy with calc; if not, write to Free Software Foundation, Inc.
+ * 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
+ *
+ * @(#) $Revision: 29.7 $
+ * @(#) $Id: zmath.h,v 29.7 2001/06/08 21:00:58 chongo Exp $
+ * @(#) $Source: /usr/local/src/cmd/calc/RCS/zmath.h,v $
+ *
+ * Under source code control:	1993/07/30 19:42:48
+ * File existed as early as:	1993
+ *
+ * Share and enjoy!  :-)	http://www.isthe.com/chongo/tech/comp/calc/
+ */
+
+/*
  * Data structure declarations for extended precision integer arithmetic.
  * The assumption made is that a long is 32 bits and shorts are 16 bits,
  * and longs must be addressible on word boundaries.
@@ -10,15 +35,24 @@
 
 
 #if !defined(__ZMATH_H__)
-#define	__ZMATH_H__
+#define __ZMATH_H__
 
 
-#include "alloc.h"
-#include "endian_calc.h"
-#include "longbits.h"
-#include "byteswap.h"
-
-#include "have_stdlib.h"
+#if defined(CALC_SRC)	/* if we are building from the calc source tree */
+# include "win32dll.h"
+# include "alloc.h"
+# include "endian_calc.h"
+# include "longbits.h"
+# include "byteswap.h"
+# include "have_stdlib.h"
+#else
+# include <calc/win32dll.h>
+# include <calc/alloc.h>
+# include <calc/endian_calc.h>
+# include <calc/longbits.h>
+# include <calc/byteswap.h>
+# include <calc/have_stdlib.h>
+#endif
 #ifdef HAVE_STDLIB_H
 # include <stdlib.h>
 #endif
@@ -31,10 +65,10 @@
 
 
 #if !defined(TRUE)
-#define	TRUE	((BOOL) 1)			/* booleans */
+#define TRUE	((BOOL) 1)			/* booleans */
 #endif
 #if !defined(FALSE)
-#define	FALSE	((BOOL) 0)
+#define FALSE	((BOOL) 0)
 #endif
 
 
@@ -89,17 +123,17 @@ typedef SB32 SFULL;			/* signed FULL */
 
 #define BASE	((FULL)1<<BASEB)		/* base for calculations */
 #define BASE1	(BASE - (FULL)1)		/* one less than base */
-#define	BASEDIG	((BASEB/16)*5)			/* number of digits in base */
+#define BASEDIG ((BASEB/16)*5)			/* number of digits in base */
 #define FULL_BITS (2*BASEB)			/* bits in a FULL */
 
-#define	TOPHALF	((FULL)1 << (BASEB-1))		/* highest bit in a HALF */
+#define TOPHALF ((FULL)1 << (BASEB-1))		/* highest bit in a HALF */
 #define MAXHALF (TOPHALF - (FULL)1)		/* largest SHALF value */
 
-#define	TOPFULL	((FULL)1 << (FULL_BITS-1))	/* highest bit in FULL */
+#define TOPFULL ((FULL)1 << (FULL_BITS-1))	/* highest bit in FULL */
 #define MAXFULL (TOPFULL - (FULL)1)		/* largest SFULL value */
 #define MAXUFULL (MAXFULL | TOPHALF)		/* largest FULL value */
 
-#define	TOPLONG	((unsigned long)1 << (LONG_BITS-1))	/* top long bit */
+#define TOPLONG ((unsigned long)1 << (LONG_BITS-1))	/* top long bit */
 #define MAXLONG ((long) (TOPLONG - (unsigned long)1))	/* largest long val */
 #define MAXULONG (MAXLONG | TOPLONG)		/* largest unsigned long val */
 
@@ -117,7 +151,7 @@ typedef FULL PRINT;			/* cast for zio printing fucctions */
 #define SWAP_B16_IN_PRINT(dest, src)	SWAP_B16_IN_FULL(dest, src)
 #define SWAP_B8_IN_PRINT(dest, src)	SWAP_B8_IN_FULL(dest, src)
 #endif
-typedef	SB32 FLAG;			/* small value (e.g. comparison) */
+typedef SB32 FLAG;			/* small value (e.g. comparison) */
 typedef SB32 BOOL;			/* TRUE or FALSE value */
 typedef SB32 LEN;			/* unit of length storage */
 
@@ -152,7 +186,7 @@ typedef SB32 LEN;			/* unit of length storage */
  * 0 hash value.  The virgin value that we use below is the hash value
  * that we would get from following 32 ASCII characters:
  *
- *              chongo <Landon Curt Noll> /\../\
+ *		chongo <Landon Curt Noll> /\../\
  *
  * Note that the \'s above are not back-slashing escape characters.
  * They are literal ASCII  backslash 0x5c characters.
@@ -191,14 +225,14 @@ typedef SB32 LEN;			/* unit of length storage */
 /*
  * LEN storage size must be <= FULL storage size
  */
-#define MAXLEN	((LEN) 0x7fffffff >> 3)	/* longest value allowed */
+#define MAXLEN	((LEN) 0x7fffffff >> 3) /* longest value allowed */
 
 
-#define	MAXREDC	5			/* number of entries in REDC cache */
-#define	SQ_ALG2	20			/* size for alternative squaring */
-#define	MUL_ALG2 20			/* size for alternative multiply */
-#define	POW_ALG2 40			/* size for using REDC for powers */
-#define	REDC_ALG2 50			/* size for using alternative REDC */
+#define MAXREDC 5			/* number of entries in REDC cache */
+#define SQ_ALG2 20			/* size for alternative squaring */
+#define MUL_ALG2 20			/* size for alternative multiply */
+#define POW_ALG2 40			/* size for using REDC for powers */
+#define REDC_ALG2 50			/* size for using alternative REDC */
 
 
 typedef union {
@@ -220,10 +254,10 @@ typedef union {
 
 #if CALC_BYTE_ORDER == LITTLE_ENDIAN
 # define silow	sis.Svalue1	/* low order half of full value */
-# define sihigh	sis.Svalue2	/* high order half of full value */
+# define sihigh sis.Svalue2	/* high order half of full value */
 #else
 # if CALC_BYTE_ORDER == BIG_ENDIAN
-#  define silow	sis.Svalue2	/* low order half of full value */
+#  define silow sis.Svalue2	/* low order half of full value */
 #  define sihigh sis.Svalue1	/* high order half of full value */
 # else
    /\oo/\    CALC_BYTE_ORDER must be BIG_ENDIAN or LITTLE_ENDIAN    /\oo/\  !!!
@@ -242,131 +276,126 @@ typedef struct {
 /*
  * Function prototypes for integer math routines.
  */
-extern HALF * alloc(LEN len);
+extern DLL HALF * alloc(LEN len);
 #ifdef	ALLOCTEST
-extern void freeh(HALF *);
+extern DLL void freeh(HALF *);
 #endif
 
 
 /*
  * Input, output, and conversion routines.
  */
-extern void zcopy(ZVALUE z, ZVALUE *res);
-extern void itoz(long i, ZVALUE *res);
-extern void utoz(FULL i, ZVALUE *res);
-extern void str2z(char *s, ZVALUE *res);
-extern long ztoi(ZVALUE z);
-extern FULL ztou(ZVALUE z);
-extern void zprintval(ZVALUE z, long decimals, long width);
-extern void zprintx(ZVALUE z, long width);
-extern void zprintb(ZVALUE z, long width);
-extern void zprinto(ZVALUE z, long width);
-extern void fitzprint(ZVALUE, long, long);
+extern DLL void zcopy(ZVALUE z, ZVALUE *res);
+extern DLL void itoz(long i, ZVALUE *res);
+extern DLL void utoz(FULL i, ZVALUE *res);
+extern DLL void str2z(char *s, ZVALUE *res);
+extern DLL long ztoi(ZVALUE z);
+extern DLL FULL ztou(ZVALUE z);
+extern DLL void zprintval(ZVALUE z, long decimals, long width);
+extern DLL void zprintx(ZVALUE z, long width);
+extern DLL void zprintb(ZVALUE z, long width);
+extern DLL void zprinto(ZVALUE z, long width);
+extern DLL void fitzprint(ZVALUE, long, long);
 
 
 /*
  * Basic numeric routines.
  */
-extern void zmuli(ZVALUE z, long n, ZVALUE *res);
-extern long zdivi(ZVALUE z, long n, ZVALUE *res);
-extern long zmodi(ZVALUE z, long n);
-extern void zadd(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern void zsub(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern void zmul(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern long zdiv(ZVALUE z1, ZVALUE z2, ZVALUE *res, ZVALUE *rem, long R);
-extern long zquo(ZVALUE z1, ZVALUE z2, ZVALUE *res, long R);
-extern long zmod(ZVALUE z1, ZVALUE z2, ZVALUE *rem, long R);
-extern void zequo(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern BOOL zdivides(ZVALUE z1, ZVALUE z2);
-extern void zor(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern void zand(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern void zxor(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern void zandnot(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern long zpopcnt(ZVALUE z, int bitval);
-extern void zshift(ZVALUE z, long n, ZVALUE *res);
-extern void zsquare(ZVALUE z, ZVALUE *res);
-extern long zlowbit(ZVALUE z);
-extern LEN zhighbit(ZVALUE z);
-extern void zbitvalue(long n, ZVALUE *res);
-extern BOOL zisset(ZVALUE z, long n);
-extern BOOL zisonebit(ZVALUE z);
-extern BOOL zisallbits(ZVALUE z);
-extern FLAG ztest(ZVALUE z);
-extern FLAG zrel(ZVALUE z1, ZVALUE z2);
-extern FLAG zabsrel(ZVALUE z1, ZVALUE z2);
-extern BOOL zcmp(ZVALUE z1, ZVALUE z2);
+extern DLL void zmuli(ZVALUE z, long n, ZVALUE *res);
+extern DLL long zdivi(ZVALUE z, long n, ZVALUE *res);
+extern DLL long zmodi(ZVALUE z, long n);
+extern DLL void zadd(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+extern DLL void zsub(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+extern DLL void zmul(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+extern DLL long zdiv(ZVALUE z1, ZVALUE z2, ZVALUE *res, ZVALUE *rem, long R);
+extern DLL long zquo(ZVALUE z1, ZVALUE z2, ZVALUE *res, long R);
+extern DLL long zmod(ZVALUE z1, ZVALUE z2, ZVALUE *rem, long R);
+extern DLL void zequo(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+extern DLL BOOL zdivides(ZVALUE z1, ZVALUE z2);
+extern DLL void zor(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+extern DLL void zand(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+extern DLL void zxor(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+extern DLL void zandnot(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+extern DLL long zpopcnt(ZVALUE z, int bitval);
+extern DLL void zshift(ZVALUE z, long n, ZVALUE *res);
+extern DLL void zsquare(ZVALUE z, ZVALUE *res);
+extern DLL long zlowbit(ZVALUE z);
+extern DLL LEN zhighbit(ZVALUE z);
+extern DLL void zbitvalue(long n, ZVALUE *res);
+extern DLL BOOL zisset(ZVALUE z, long n);
+extern DLL BOOL zisonebit(ZVALUE z);
+extern DLL BOOL zisallbits(ZVALUE z);
+extern DLL FLAG ztest(ZVALUE z);
+extern DLL FLAG zrel(ZVALUE z1, ZVALUE z2);
+extern DLL FLAG zabsrel(ZVALUE z1, ZVALUE z2);
+extern DLL BOOL zcmp(ZVALUE z1, ZVALUE z2);
 
 
 /*
  * More complicated numeric functions.
  */
-extern FULL uugcd(FULL i1, FULL i2);
-extern long iigcd(long i1, long i2);
-extern void zgcd(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern void zlcm(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern void zreduce(ZVALUE z1, ZVALUE z2, ZVALUE *z1res, ZVALUE *z2res);
-extern void zfact(ZVALUE z, ZVALUE *dest);
-extern void zperm(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern void zcomb(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern FLAG zjacobi(ZVALUE z1, ZVALUE z2);
-extern void zfib(ZVALUE z, ZVALUE *res);
-extern void zpowi(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern void ztenpow(long power, ZVALUE *res);
-extern void zpowermod(ZVALUE z1, ZVALUE z2, ZVALUE z3, ZVALUE *res);
-extern BOOL zmodinv(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern BOOL zrelprime(ZVALUE z1, ZVALUE z2);
-extern long zlog(ZVALUE z1, ZVALUE z2);
-extern long zlog10(ZVALUE z);
-extern long zdivcount(ZVALUE z1, ZVALUE z2);
-extern long zfacrem(ZVALUE z1, ZVALUE z2, ZVALUE *rem);
-extern void zgcdrem(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern long zdigits(ZVALUE z1);
-extern long zdigit(ZVALUE z1, long n);
-extern FLAG zsqrt(ZVALUE z1, ZVALUE *dest, long R);
-extern void zroot(ZVALUE z1, ZVALUE z2, ZVALUE *dest);
-extern BOOL zissquare(ZVALUE z);
-extern void zhnrmod(ZVALUE v, ZVALUE h, ZVALUE zn, ZVALUE zr, ZVALUE *res);
+extern DLL FULL uugcd(FULL i1, FULL i2);
+extern DLL long iigcd(long i1, long i2);
+extern DLL void zgcd(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+extern DLL void zlcm(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+extern DLL void zreduce(ZVALUE z1, ZVALUE z2, ZVALUE *z1res, ZVALUE *z2res);
+extern DLL void zfact(ZVALUE z, ZVALUE *dest);
+extern DLL void zperm(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+extern DLL int zcomb(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+extern DLL FLAG zjacobi(ZVALUE z1, ZVALUE z2);
+extern DLL void zfib(ZVALUE z, ZVALUE *res);
+extern DLL void zpowi(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+extern DLL void ztenpow(long power, ZVALUE *res);
+extern DLL void zpowermod(ZVALUE z1, ZVALUE z2, ZVALUE z3, ZVALUE *res);
+extern DLL BOOL zmodinv(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+extern DLL BOOL zrelprime(ZVALUE z1, ZVALUE z2);
+extern DLL long zlog(ZVALUE z1, ZVALUE z2);
+extern DLL long zlog10(ZVALUE z);
+extern DLL long zdivcount(ZVALUE z1, ZVALUE z2);
+extern DLL long zfacrem(ZVALUE z1, ZVALUE z2, ZVALUE *rem);
+extern DLL long zgcdrem(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+extern DLL long zdigits(ZVALUE z1);
+extern DLL long zdigit(ZVALUE z1, long n);
+extern DLL FLAG zsqrt(ZVALUE z1, ZVALUE *dest, long R);
+extern DLL void zroot(ZVALUE z1, ZVALUE z2, ZVALUE *dest);
+extern DLL BOOL zissquare(ZVALUE z);
+extern DLL void zhnrmod(ZVALUE v, ZVALUE h, ZVALUE zn, ZVALUE zr, ZVALUE *res);
 
 
 /*
  * Prime related functions.
  */
-extern FLAG zisprime(ZVALUE z);
-extern FULL znprime(ZVALUE z);
-extern FULL next_prime(FULL v);
-extern FULL zpprime(ZVALUE z);
-extern void zpfact(ZVALUE z, ZVALUE *dest);
-extern BOOL zprimetest(ZVALUE z, long count, ZVALUE skip);
-extern BOOL zredcprimetest(ZVALUE z, long count, ZVALUE skip);
-extern BOOL znextcand(ZVALUE z1, long count, ZVALUE skip, ZVALUE res, ZVALUE mod, ZVALUE *cand);
-extern BOOL zprevcand(ZVALUE z1, long count, ZVALUE skip, ZVALUE res, ZVALUE mod, ZVALUE *cand);
-extern FULL zlowfactor(ZVALUE z, long count);
-extern FLAG zfactor(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern long zpix(ZVALUE z1);
-extern void zlcmfact(ZVALUE z, ZVALUE *dest);
+extern DLL FLAG zisprime(ZVALUE z);
+extern DLL FULL znprime(ZVALUE z);
+extern DLL FULL next_prime(FULL v);
+extern DLL FULL zpprime(ZVALUE z);
+extern DLL void zpfact(ZVALUE z, ZVALUE *dest);
+extern DLL BOOL zprimetest(ZVALUE z, long count, ZVALUE skip);
+extern DLL BOOL zredcprimetest(ZVALUE z, long count, ZVALUE skip);
+extern DLL BOOL znextcand(ZVALUE z1, long count, ZVALUE skip, ZVALUE res, ZVALUE mod, ZVALUE *cand);
+extern DLL BOOL zprevcand(ZVALUE z1, long count, ZVALUE skip, ZVALUE res, ZVALUE mod, ZVALUE *cand);
+extern DLL FULL zlowfactor(ZVALUE z, long count);
+extern DLL FLAG zfactor(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+extern DLL long zpix(ZVALUE z1);
+extern DLL void zlcmfact(ZVALUE z, ZVALUE *dest);
 
 
 /*
  * Misc misc functions. :-)
  */
-#if 0
-extern void zapprox(ZVALUE z1, ZVALUE z2, ZVALUE *res1, ZVALUE *res2);
-extern void zmulmod(ZVALUE z1, ZVALUE z2, ZVALUE z3, ZVALUE *res);
-extern void zsubmod(ZVALUE z1, ZVALUE z2, ZVALUE z3, ZVALUE *res);
-#endif
-extern void zsquaremod(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern void zminmod(ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern BOOL zcmpmod(ZVALUE z1, ZVALUE z2, ZVALUE z3);
-extern void zio_init(void);
+extern DLL void zsquaremod(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+extern DLL void zminmod(ZVALUE z1, ZVALUE z2, ZVALUE *res);
+extern DLL BOOL zcmpmod(ZVALUE z1, ZVALUE z2, ZVALUE z3);
+extern DLL void zio_init(void);
 
 
 /*
  * These functions are for internal use only.
  */
-extern void ztrim(ZVALUE *z);
-extern void zshiftr(ZVALUE z, long n);
-extern void zshiftl(ZVALUE z, long n);
-extern HALF *zalloctemp(LEN len);
+extern DLL void ztrim(ZVALUE *z);
+extern DLL void zshiftr(ZVALUE z, long n);
+extern DLL void zshiftl(ZVALUE z, long n);
+extern DLL HALF *zalloctemp(LEN len);
 
 
 /*
@@ -376,20 +405,20 @@ extern HALF *zalloctemp(LEN len);
  * calculations with more than one modulus at the same time.
  * Len of zero means the structure is not initialized.
  */
-typedef	struct {
+typedef struct {
 	LEN len;		/* number of words in binary modulus */
 	ZVALUE mod;		/* modulus REDC is computing with */
 	ZVALUE inv;		/* inverse of modulus in binary modulus */
 	ZVALUE one;		/* REDC format for the number 1 */
 } REDC;
 
-extern REDC *zredcalloc(ZVALUE z1);
-extern void zredcfree(REDC *rp);
-extern void zredcencode(REDC *rp, ZVALUE z1, ZVALUE *res);
-extern void zredcdecode(REDC *rp, ZVALUE z1, ZVALUE *res);
-extern void zredcmul(REDC *rp, ZVALUE z1, ZVALUE z2, ZVALUE *res);
-extern void zredcsquare(REDC *rp, ZVALUE z1, ZVALUE *res);
-extern void zredcpower(REDC *rp, ZVALUE z1, ZVALUE z2, ZVALUE *res);
+extern DLL REDC *zredcalloc(ZVALUE z1);
+extern DLL void zredcfree(REDC *rp);
+extern DLL void zredcencode(REDC *rp, ZVALUE z1, ZVALUE *res);
+extern DLL void zredcdecode(REDC *rp, ZVALUE z1, ZVALUE *res);
+extern DLL void zredcmul(REDC *rp, ZVALUE z1, ZVALUE z2, ZVALUE *res);
+extern DLL void zredcsquare(REDC *rp, ZVALUE z1, ZVALUE *res);
+extern DLL void zredcpower(REDC *rp, ZVALUE z1, ZVALUE z2, ZVALUE *res);
 
 
 /*
@@ -441,15 +470,15 @@ extern void zredcpower(REDC *rp, ZVALUE z1, ZVALUE z2, ZVALUE *res);
 /*
  * Some algorithms testing for values of a certain length.  Macros such as
  * zistiny() do this well.  In other cases algorthms require tests for values
- * in comparison to a given power of 2.  In the later case, zistiny() compares
- * against a different power of 2 on a 64 bit machine.  The macros below
+ * in comparison to a given power of 2.	 In the later case, zistiny() compares
+ * against a different power of 2 on a 64 bit machine.	The macros below
  * provide a tests against powers of 2 that are independent of the work size.
  *
- * 	zge16b(z)	TRUE if abs(z) >= 2^16
- * 	zge24b(z)	TRUE if abs(z) >= 2^24
- * 	zge31b(z)	TRUE if abs(z) >= 2^31
- * 	zge32b(z)	TRUE if abs(z) >= 2^32
- * 	zge64b(z)	TRUE if abs(z) >= 2^64
+ *	zge16b(z)	TRUE if abs(z) >= 2^16
+ *	zge24b(z)	TRUE if abs(z) >= 2^24
+ *	zge31b(z)	TRUE if abs(z) >= 2^31
+ *	zge32b(z)	TRUE if abs(z) >= 2^32
+ *	zge64b(z)	TRUE if abs(z) >= 2^64
  */
 #if BASEB == 32
 
@@ -504,11 +533,11 @@ extern void zredcpower(REDC *rp, ZVALUE z1, ZVALUE z2, ZVALUE *res);
  */
 #define ztolong(z)	((long)(ztoulong(z) & MAXLONG))
 
-#define	zclearval(z)	memset((z).v, 0, (z).len * sizeof(HALF))
-#define	zcopyval(z1,z2)	memcpy((z2).v, (z1).v, (z1).len * sizeof(HALF))
+#define zclearval(z)	memset((z).v, 0, (z).len * sizeof(HALF))
+#define zcopyval(z1,z2) memcpy((z2).v, (z1).v, (z1).len * sizeof(HALF))
 #define zquicktrim(z)	{if (((z).len > 1) && ((z).v[(z).len-1] == 0)) \
 				(z).len--;}
-#define	zfree(z)	freeh((z).v)
+#define zfree(z)	freeh((z).v)
 
 
 /*
@@ -530,32 +559,32 @@ extern void zredcpower(REDC *rp, ZVALUE z1, ZVALUE z2, ZVALUE *res);
 /*
  * Output routines for either FILE handles or strings.
  */
-extern void math_chr(int ch);
-extern void math_str(char *str);
-extern void math_fill(char *str, long width);
-extern void math_flush(void);
-extern void math_divertio(void);
-extern void math_cleardiversions(void);
-extern char *math_getdivertedio(void);
-extern int math_setmode(int mode);
-extern long math_setdigits(long digits);
-extern void math_fmt(char *, ...);
+extern DLL void math_chr(int ch);
+extern DLL void math_str(char *str);
+extern DLL void math_fill(char *str, long width);
+extern DLL void math_flush(void);
+extern DLL void math_divertio(void);
+extern DLL void math_cleardiversions(void);
+extern DLL char *math_getdivertedio(void);
+extern DLL int math_setmode(int mode);
+extern DLL LEN math_setdigits(LEN digits);
+extern DLL void math_fmt(char *, ...);
 
 
 /*
  * The error routine.
  */
-extern void math_error(char *, ...);
+extern DLL void math_error(char *, ...);
 
 
 /*
  * external swap functions
  */
-extern HALF *swap_b8_in_HALFs(HALF *dest, HALF *src, LEN len);
-extern ZVALUE *swap_b8_in_ZVALUE(ZVALUE *dest, ZVALUE *src, BOOL all);
-extern HALF *swap_b16_in_HALFs(HALF *dest, HALF *src, LEN len);
-extern ZVALUE *swap_b16_in_ZVALUE(ZVALUE *dest, ZVALUE *src, BOOL all);
-extern ZVALUE *swap_HALF_in_ZVALUE(ZVALUE *dest, ZVALUE *src, BOOL all);
+extern DLL HALF *swap_b8_in_HALFs(HALF *dest, HALF *src, LEN len);
+extern DLL ZVALUE *swap_b8_in_ZVALUE(ZVALUE *dest, ZVALUE *src, BOOL all);
+extern DLL HALF *swap_b16_in_HALFs(HALF *dest, HALF *src, LEN len);
+extern DLL ZVALUE *swap_b16_in_ZVALUE(ZVALUE *dest, ZVALUE *src, BOOL all);
+extern DLL ZVALUE *swap_HALF_in_ZVALUE(ZVALUE *dest, ZVALUE *src, BOOL all);
 
 
 /*
@@ -575,7 +604,7 @@ extern ZVALUE _sqbase_, _pow4base_, _pow8base_;
 
 extern ZVALUE _b32_, _b64_;
 
-extern BOOL _math_abort_;	/* nonzero to abort calculations */
+extern DLL BOOL _math_abort_;	/* nonzero to abort calculations */
 extern ZVALUE _tenpowers_[];	/* table of 10^2^n */
 
 /*
@@ -631,7 +660,7 @@ typedef struct {
 
 #else
 
-   /\../\	FULL_BITS must be 32 or 64	/\../\   !!!
+   /\../\	FULL_BITS must be 32 or 64	/\../\	 !!!
 
 #endif
 
